@@ -9,7 +9,9 @@ from compas_cem.equilibrium.force_numpy import equilibrium_state_numpy
 from compas_cem.optimization import nlopt_solver
 from compas_cem.optimization import grad_autograd
 from compas_cem.optimization import objective_function_numpy
+from compas_cem.optimization import nlopt_status
 
+from nlopt import RoundoffLimited
 
 __all__ = ["Optimizer"]
 
@@ -28,6 +30,8 @@ class Optimizer():
         self.x_opt = None
         self.penalty = None
         self.evals = None
+        self.gradient_norm = None
+        self.status = None
 
 # ------------------------------------------------------------------------------
 # Counters
@@ -103,29 +107,29 @@ class Optimizer():
 # Objective Function
 # ------------------------------------------------------------------------------
 
-    def objective_func(self, topology, grad_func):
+    def objective_func(self, topology, grad_func, tmax, eta):
         """
         Test with autodiff.
         """
         obj_func = objective_function_numpy
-        func = partial(self._optimize_form, topology=topology)
+        func = partial(self._optimize_form, topology=topology, tmax=tmax, eta=eta)
         return partial(obj_func, x_func=func, grad_func=grad_func)
 
 # ------------------------------------------------------------------------------
 # Gradient Function
 # ------------------------------------------------------------------------------
 
-    def gradient_func(self, topology):
+    def gradient_func(self, topology, tmax, eta):
         """
         """
-        x_func = partial(self._optimize_form, topology=topology)
+        x_func = partial(self._optimize_form, topology=topology, tmax=tmax, eta=eta)
         return partial(grad_autograd, grad_func=x_func)
 
 # ------------------------------------------------------------------------------
 # Solver
 # ------------------------------------------------------------------------------
 
-    def solve_nlopt(self, topology, algorithm, iters, eps=None):
+    def solve_nlopt(self, topology, algorithm, iters, eps=None, tmax=100, eta=1e-6):
         """
         Solve an optimization problem with NLOpt.
         """
@@ -133,8 +137,9 @@ class Optimizer():
         self.check_optimization_sanity()
 
         # compose gradient and objective functions
-        grad_func = self.gradient_func(topology.copy())
-        penalty_func = self.objective_func(topology, grad_func)
+        topology_b = topology.copy()
+        grad_func = self.gradient_func(topology_b, tmax, eta)
+        penalty_func = self.objective_func(topology, grad_func, tmax, eta)
 
         # generate optimization variables
         x = self.optimization_parameters(topology)
@@ -156,24 +161,34 @@ class Optimizer():
         solver = nlopt_solver(**hyper_parameters)
 
         # solve optimization problem
+        x_opt = None
         try:
             x_opt = solver.optimize(x)
             evals = solver.get_numevals()
             print("Optimization ended correctly!")
             print("Number of evaluations incurred: {}".format(evals))
-        except RuntimeError:
-            evals = solver.get_numevals()
-            print("Tol unreached and max iters exhausted! Try increasing them.")
-            print("Number of evaluations incurred: {}".format(evals))
-            x_opt = None
+        except RoundoffLimited:
+            print("Optimization was halted because roundoff errors limited progress")
+            print("Returned results are generally still useful though!")
+            x_opt = self.optimization_parameters(topology)
 
         # fetch last optimum value of loss function
         loss_opt = solver.last_optimum_value()
+        evals = solver.get_numevals()
+        status = nlopt_status(solver.last_optimize_result())
 
         # set optimizer attributes
         self.x_opt = x_opt
         self.penalty = loss_opt
         self.evals = evals
+        self.status = status
+
+        # set norm of the gradient
+        # np.zeros is a dummy array (signature requirement set by nlopt)
+        self.gradient = grad_func(x_opt, np.zeros(x_opt.size))
+        self.gradient_norm = np.linalg.norm(self.gradient)
+
+        print("Optimization status: {}".format(status))
 
         # exit like a champion
         return static_equilibrium(topology)
@@ -276,13 +291,13 @@ class Optimizer():
 
         return penalty
 
-    def _optimize_form(self, parameters, topology):
+    def _optimize_form(self, parameters, topology, tmax, eta):
         """
         """
         self._update_topology_origin_nodes(parameters, topology)
         self._update_topology_edges(parameters, topology)
 
-        eq_state = equilibrium_state_numpy(topology, tmax=100, eta=1e-5)
+        eq_state = equilibrium_state_numpy(topology, tmax, eta)
 
         return self._calculate_penalty(eq_state)
 
